@@ -35,7 +35,7 @@ void Player::thread_func(AVFormatContext *file_avctx)
 	QOpenGLContext *context = create_context(surface);
 	if (!make_current(context, surface)) {
 		printf("oops\n");
-		exit(1);
+		abort();
 	}
 
 	check_error();
@@ -139,7 +139,14 @@ void Player::play_playlist_once()
 	}
 
 	steady_clock::duration time_slept = steady_clock::now() - before_sleep;
-	pts += duration_cast<duration<size_t, TimebaseRatio>>(time_slept).count();
+	int64_t slept_pts = duration_cast<duration<size_t, TimebaseRatio>>(time_slept).count();
+	if (slept_pts > 0) {
+		if (video_stream != nullptr) {
+			// Add silence for the time we're waiting.
+			video_stream->schedule_silence(steady_clock::now(), pts, slept_pts, QueueSpotHolder());
+		}
+		pts += slept_pts;
+	}
 
 	if (!clip_ready) {
 		if (video_stream != nullptr) {
@@ -205,6 +212,10 @@ void Player::play_playlist_once()
 				break;
 			}
 
+			// Only play audio if we're within 0.1% of normal speed. We could do
+			// stretching or pitch shift later if it becomes needed.
+			bool play_audio = clip->speed * master_speed >= 0.999 && clip->speed * master_speed <= 1.001;
+
 			{
 				lock_guard<mutex> lock(queue_state_mu);
 				if (splice_ready) {
@@ -349,7 +360,7 @@ void Player::play_playlist_once()
 			if (frame_lower.pts == frame_upper.pts || global_flags.interpolation_quality == 0 || video_stream == nullptr) {
 				display_single_frame(primary_stream_idx, frame_lower, secondary_stream_idx,
 				                     secondary_frame, fade_alpha, next_frame_start, /*snapped=*/false,
-				                     subtitle);
+				                     subtitle, play_audio);
 				continue;
 			}
 
@@ -362,7 +373,7 @@ void Player::play_playlist_once()
 				if (fabs(snap_frame.pts - in_pts) < pts_snap_tolerance) {
 					display_single_frame(primary_stream_idx, snap_frame, secondary_stream_idx,
 					                     secondary_frame, fade_alpha, next_frame_start, /*snapped=*/true,
-					                     subtitle);
+					                     subtitle, play_audio);
 					in_pts_origin += snap_frame.pts - in_pts;
 					snapped = true;
 					break;
@@ -418,7 +429,7 @@ void Player::play_playlist_once()
 			video_stream->schedule_interpolated_frame(
 				next_frame_start, pts, display_func, QueueSpotHolder(this),
 				frame_lower, frame_upper, alpha,
-				secondary_frame, fade_alpha, subtitle);
+				secondary_frame, fade_alpha, subtitle, play_audio);
 			last_pts_played = in_pts;  // Not really needed; only previews use last_pts_played.
 		}
 
@@ -439,7 +450,7 @@ void Player::play_playlist_once()
 	}
 }
 
-void Player::display_single_frame(int primary_stream_idx, const FrameOnDisk &primary_frame, int secondary_stream_idx, const FrameOnDisk &secondary_frame, double fade_alpha, steady_clock::time_point frame_start, bool snapped, const std::string &subtitle)
+void Player::display_single_frame(int primary_stream_idx, const FrameOnDisk &primary_frame, int secondary_stream_idx, const FrameOnDisk &secondary_frame, double fade_alpha, steady_clock::time_point frame_start, bool snapped, const std::string &subtitle, bool play_audio)
 {
 	auto display_func = [this, primary_stream_idx, primary_frame, secondary_frame, fade_alpha] {
 		if (destination != nullptr) {
@@ -458,7 +469,7 @@ void Player::display_single_frame(int primary_stream_idx, const FrameOnDisk &pri
 			}
 			video_stream->schedule_original_frame(
 				frame_start, pts, display_func, QueueSpotHolder(this),
-				primary_frame, subtitle);
+				primary_frame, subtitle, play_audio);
 		} else {
 			assert(secondary_frame.pts != -1);
 			// NOTE: We could be increasing unused metrics for previews, but that's harmless.
